@@ -1,3 +1,33 @@
+/* Copyright (c) 2000 Shlomi Fish
+ *
+ * Permission is hereby granted, free of charge, to any person
+ * obtaining a copy of this software and associated documentation
+ * files (the "Software"), to deal in the Software without
+ * restriction, including without limitation the rights to use,
+ * copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following
+ * conditions:
+ *
+ * The above copyright notice and this permission notice shall be
+ * included in all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
+ * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+ * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
+ * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
+ * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+ * OTHER DEALINGS IN THE SOFTWARE.
+ */
+/*
+ * cmd_line.c - the Freecell Solver command line arguments-like parsing 
+ * routines. Useful for more easily configuring a Freecell Solver instance.
+ */
+
+#define BUILDING_DLL 1
+
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -5,28 +35,34 @@
 
 #include "fcs_user.h"
 #include "fcs_cl.h"
-#include "cl_chop.h"
+#include "split_cmd_line.h"
 
 #include "prefix.h"
-
-#ifdef DMALLOC
-#include "dmalloc.h"
-#endif
+#include "inline.h"
 
 #include "cmd_line_enum.h"
 
-static int read_preset(char * preset_name, args_man_t * * args, char * * opened_files_dir_to_assign, char * user_preset_dir)
+static GCC_INLINE int read_preset(char * preset_name, args_man_t * * args_man, char * * opened_files_dir_to_assign, char * user_preset_dir)
 {
     int ret_code = 1;
     char * home_dir_presetrc = NULL, * global_presetrc = NULL, * env_var_presetrc = NULL;
     char * path;
-    char * * presetrc_pathes[5] = {&env_var_presetrc, &home_dir_presetrc, &global_presetrc, &user_preset_dir, NULL};
+    char * * presetrc_pathes[5];
     int path_idx;
     char line[8192];
     FILE * f = NULL;
-    char * fgets_ret;
     char * opened_files_dir = NULL;
     int read_next_preset = 0;
+   
+
+    {
+        register int idx = 0;
+        presetrc_pathes[idx++] = &env_var_presetrc;
+        presetrc_pathes[idx++] = &home_dir_presetrc;
+        presetrc_pathes[idx++] = &global_presetrc;
+        presetrc_pathes[idx++] = &user_preset_dir;
+        presetrc_pathes[idx++] = NULL;
+    }
 
     {
         char * home_dir;
@@ -57,8 +93,7 @@ static int read_preset(char * preset_name, args_man_t * * args, char * * opened_
         }
         while(1)
         {
-            fgets_ret = fgets(line, sizeof(line), f);
-            if (fgets_ret == NULL)
+            if (fgets(line, sizeof(line), f) == NULL)
             {
                 break;
             }
@@ -94,10 +129,10 @@ static int read_preset(char * preset_name, args_man_t * * args, char * * opened_
             {
                 if (read_next_preset)
                 {
-                    *args = fc_solve_args_man_alloc();
-                    fc_solve_args_man_chop(*args, line+8);
+                    *args_man = fc_solve_args_man_alloc();
+                    fc_solve_args_man_chop(*args_man, line+8);
                     ret_code = 0;
-                    goto HAVE_PRESET;
+                    goto have_preset;
                 }
             }
         }
@@ -105,7 +140,7 @@ static int read_preset(char * preset_name, args_man_t * * args, char * * opened_
         f = NULL;
 #undef nullify_newline
     }
-HAVE_PRESET:
+have_preset:
 
     if (f)
     {
@@ -133,7 +168,7 @@ HAVE_PRESET:
 }
 
 
-int freecell_solver_user_cmd_line_parse_args_with_file_nesting_count(
+DLLEXPORT int freecell_solver_user_cmd_line_parse_args_with_file_nesting_count(
     void * instance,
     int argc,
     char * argv[],
@@ -151,7 +186,6 @@ int freecell_solver_user_cmd_line_parse_args_with_file_nesting_count(
     char * * known_param;
     int num_to_skip;
     int callback_ret;
-    int ret;
     int opt;
     const char * p;
 
@@ -169,6 +203,8 @@ int freecell_solver_user_cmd_line_parse_args_with_file_nesting_count(
         }
         if ((*known_param) != NULL )
         {
+            int ret;
+
             callback_ret = callback(instance, argc, argv, arg-&(argv[0]), &num_to_skip, &ret, callback_context);
             if (callback_ret == FCS_CMD_LINE_SKIP)
             {
@@ -349,6 +385,17 @@ opt = FCS_OPT_NEXT_HARD_THREAD;
 
 break;
 
+case 'i':
+{
+if (!strncmp(p, "nstance", 7)) {
+p += 7;
+opt = FCS_OPT_NEXT_INSTANCE;
+
+}
+}
+
+break;
+
 case 's':
 {
 if (!strncmp(p, "oft-thread", 10)) {
@@ -697,6 +744,18 @@ opt = FCS_OPT_NEXT_HARD_THREAD;
 
 break;
 
+case 'i':
+
+{
+if (*p == '\0')
+{
+
+opt = FCS_OPT_NEXT_INSTANCE;
+}
+}
+
+break;
+
 case 's':
 {
 if (!strncmp(p, "t", 1)) {
@@ -830,6 +889,7 @@ break;
 
         case FCS_OPT_TESTS_ORDER: /* STRINGS=-to|--tests-order; */
         {
+            int ret;
             char * fcs_user_errstr;
 
             PROCESS_OPT_ARG() ;
@@ -942,19 +1002,12 @@ break;
 
         case FCS_OPT_SEQUENCE_MOVE: /* STRINGS=--sequence-move; */
         {
-            int unlimited;
-
             PROCESS_OPT_ARG() ;
 
-            if (!strcmp((*arg), "unlimited"))
-            {
-                unlimited = 1;
-            }
-            else
-            {
-                unlimited = 0;
-            }
-            freecell_solver_user_set_sequence_move(instance, unlimited);
+            freecell_solver_user_set_sequence_move(
+                instance,
+                !strcmp((*arg), "unlimited")
+            );
         }
         break;
 
@@ -985,6 +1038,8 @@ break;
 
         case FCS_OPT_GAME: /* STRINGS=--game|--preset|-g; */
         {
+            int ret;
+
             PROCESS_OPT_ARG() ;
 
             ret = freecell_solver_user_apply_preset(instance, (*arg));
@@ -1159,6 +1214,11 @@ break;
         }
         break;
 
+        case FCS_OPT_NEXT_INSTANCE: /* STRINGS=-ni|--next-instance; */
+        {
+            freecell_solver_user_next_instance(instance);
+        }
+
         case FCS_OPT_NEXT_SOFT_THREAD: /* STRINGS=-nst|--next-soft-thread; */
         case FCS_OPT_NEXT_HARD_THREAD: /* STRINGS=-nht|--next-hard-thread; */
         {
@@ -1229,6 +1289,7 @@ break;
 
         case FCS_OPT_OPTIMIZATION_TESTS_ORDER: /* STRINGS=-opt-to|--optimization-tests-order; */
         {
+            int ret;
             char * fcs_user_errstr;
 
             PROCESS_OPT_ARG() ;
@@ -1305,13 +1366,15 @@ break;
             }
             else
             {
-                int num_to_skip = 0;
                 char * s, * buffer;
                 FILE * f;
                 long file_len;
                 int ret;
                 size_t num_read;
                 args_man_t * args_man;
+                int num_file_args_to_skip;
+
+                num_file_args_to_skip = 0;
 
                 s = (*arg);
                 while(isdigit(*s))
@@ -1320,15 +1383,20 @@ break;
                 }
                 if (*s == ',')
                 {
-                    num_to_skip = atoi((*arg));
+                    num_file_args_to_skip = atoi((*arg));
                     s++;
                 }
 
                 if (opened_files_dir)
                 {
                     char * complete_path;
+                    register int len;
 
-                    complete_path = malloc(strlen(opened_files_dir)+strlen(s)+1);
+                    len = strlen(opened_files_dir);
+                    len += strlen(s);
+                    len++;
+
+                    complete_path = malloc(len);
                     sprintf(complete_path, "%s%s", opened_files_dir, s);
                     f = fopen(complete_path, "rt");
                     free(complete_path);
@@ -1388,7 +1456,7 @@ break;
                     RET_ERROR_IN_ARG() ;
                 }
 
-                if (num_to_skip >= args_man->argc)
+                if (num_file_args_to_skip >= args_man->argc)
                 {
                     /* Do nothing */
                 }
@@ -1396,8 +1464,8 @@ break;
                 {
                     ret = freecell_solver_user_cmd_line_parse_args_with_file_nesting_count(
                         instance,
-                        args_man->argc - num_to_skip,
-                        args_man->argv + num_to_skip,
+                        args_man->argc - num_file_args_to_skip,
+                        args_man->argv + num_file_args_to_skip,
                         0,
                         known_parameters,
                         callback,
@@ -1444,6 +1512,8 @@ break;
                 }
                 else
                 {
+                    int ret;
+
                     ret = freecell_solver_user_cmd_line_parse_args_with_file_nesting_count(
                         instance,
                         preset_args->argc,
@@ -1486,7 +1556,7 @@ break;
     return FCS_CMD_LINE_OK;
 }
 
-int freecell_solver_user_cmd_line_parse_args(
+DLLEXPORT int freecell_solver_user_cmd_line_parse_args(
     void * instance,
     int argc,
     char * argv[],
