@@ -1,14 +1,35 @@
+/* Copyright (c) 2000 Shlomi Fish
+ *
+ * Permission is hereby granted, free of charge, to any person
+ * obtaining a copy of this software and associated documentation
+ * files (the "Software"), to deal in the Software without
+ * restriction, including without limitation the rights to use,
+ * copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following
+ * conditions:
+ *
+ * The above copyright notice and this permission notice shall be
+ * included in all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
+ * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+ * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
+ * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
+ * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+ * OTHER DEALINGS IN THE SOFTWARE.
+ */
 /*
  * fcs_hash.c - an implementation of a simplistic (keys only) hash. This
  * hash uses chaining and re-hashing and was found to be very fast. Not all
  * of the functions of the hash ADT are implemented, but it is useful enough
  * for Freecell Solver.
  *
- * Written by Shlomi Fish ( http://www.shlomifish.org/ ), 2000
- *
- * This file is in the public domain (it's uncopyrighted).
  */
 
+#define BUILDING_DLL 1
 #include "config.h"
 
 #if (FCS_STATE_STORAGE == FCS_STATE_STORAGE_INTERNAL_HASH) || (defined(INDIRECT_STACK_STATES) && (FCS_STACK_STORAGE == FCS_STACK_STORAGE_INTERNAL_HASH))
@@ -20,68 +41,72 @@
 
 #include "alloc.h"
 
-#ifdef DMALLOC
-#include "dmalloc.h"
-#endif
+#include "inline.h"
 
-static void SFO_hash_rehash(SFO_hash_t * hash);
+static void GCC_INLINE fc_solve_hash_rehash(fc_solve_hash_t * hash);
 
 
 
-SFO_hash_t * fc_solve_hash_init(
-    SFO_hash_value_t wanted_size,
+void fc_solve_hash_init(
+    fc_solve_hash_t * hash,
+    fc_solve_hash_value_t wanted_size,
+#ifdef FCS_WITH_CONTEXT_VARIABLE
     int (*compare_function)(const void * key1, const void * key2, void * context),
     void * context
+#else
+    int (*compare_function)(const void * key1, const void * key2)
+#endif
     )
 {
     int size;
-    SFO_hash_t * hash;
-
-    /* Find a prime number that is greater than the initial wanted size */
+    /* Find a size that's a power of 2 that's just greater than 
+     * the wanted_size. */
     size = 256;
     while (size < wanted_size)
     {
         size <<= 1;
     }
 
-    hash = (SFO_hash_t *)malloc(sizeof(SFO_hash_t));
-
     hash->size = size;
     hash->size_bitmask = size-1;
 
     hash->num_elems = 0;
 
     /* Allocate a table of size entries */
-    hash->entries = (SFO_hash_symlink_t *)malloc(
-        sizeof(SFO_hash_symlink_t) * size
+    hash->entries = (fc_solve_hash_symlink_t *)malloc(
+        sizeof(fc_solve_hash_symlink_t) * size
         );
 
     hash->compare_function = compare_function;
+#ifdef FCS_WITH_CONTEXT_VARIABLE
     hash->context = context;
+#endif
 
     /* Initialize all the cells of the hash table to NULL, which indicate
        that the cork of the linked list is right at the start */
-    memset(hash->entries, 0, sizeof(SFO_hash_symlink_t)*size);
+    memset(hash->entries, 0, sizeof(fc_solve_hash_symlink_t)*size);
 
-    hash->allocator = fc_solve_compact_allocator_new();
+    fc_solve_compact_allocator_init(&(hash->allocator));
 
-    return hash;
+    return;
 }
 
 int fc_solve_hash_insert(
-    SFO_hash_t * hash,
+    fc_solve_hash_t * hash,
     void * key,
     void * val,
     void * * existing_key,
     void * * existing_val,
-    SFO_hash_value_t hash_value,
-    SFO_hash_value_t secondary_hash_value,
-    int optimize_for_caching
+    fc_solve_hash_value_t hash_value
+#ifdef FCS_ENABLE_SECONDARY_HASH_VALUE
+    , fc_solve_hash_value_t secondary_hash_value
+#endif
     )
 {
     int place;
-    SFO_hash_symlink_t * list;
-    SFO_hash_symlink_item_t * item, * last_item;
+    fc_solve_hash_symlink_t * list;
+    fc_solve_hash_symlink_item_t * item, * last_item;
+    fc_solve_hash_symlink_item_t * * item_placeholder;
 
     /* Get the index of the appropriate chain in the hash table */
     place = hash_value & (hash->size_bitmask);
@@ -91,15 +116,8 @@ int fc_solve_hash_insert(
     if (list->first_item == NULL)
     {
         /* Allocate a first item with that key/val pair */
-        fcs_compact_alloc_into_var(item, hash->allocator, SFO_hash_symlink_item_t);
-        list->first_item = item;
-        item->next = NULL;
-        item->key = key;
-        item->val = val;
-        item->hash_value = hash_value;
-        item->secondary_hash_value = secondary_hash_value;
-
-        goto rehash_check;
+        item_placeholder = &(list->first_item);
+        goto alloc_item;
     }
 
     /* Initialize item to the chain's first_item */
@@ -115,24 +133,16 @@ int fc_solve_hash_insert(
         */
         if (
             (item->hash_value == hash_value) &&
+#ifdef FCS_ENABLE_SECONDARY_HASH_VALUE
             (item->secondary_hash_value == secondary_hash_value) &&
-            (!(hash->compare_function(item->key, key, hash->context)))
+#endif
+            (!(hash->compare_function(item->key, key
+#ifdef FCS_WITH_CONTEXT_VARIABLE
+                                      , hash->context
+#endif
+               )))
            )
         {
-            if (optimize_for_caching)
-            {
-                /*
-                 * Place the item in the beginning of the chain.
-                 * If last_item == NULL it is already the first item so leave
-                 * it alone
-                 * */
-                if (last_item != NULL)
-                {
-                    last_item->next = item->next;
-                    item->next = list->first_item;
-                    list->first_item = item;
-                }
-            }
             *existing_key = item->key;
             *existing_val = item->val;
 
@@ -144,36 +154,22 @@ int fc_solve_hash_insert(
         item = item->next;
     }
 
-    if (optimize_for_caching)
-    {
-        /* Put the new element at the beginning of the list */
-        fcs_compact_alloc_into_var(item, hash->allocator, SFO_hash_symlink_item_t);
-        item->next = list->first_item;
-        item->key = key;
-        item->val = val;
-        item->hash_value = hash_value;
-        list->first_item = item;
-        item->secondary_hash_value = secondary_hash_value;
-    }
-    else
-    {
-        /* Put the new element at the end of the list */
-        fcs_compact_alloc_into_var(item, hash->allocator, SFO_hash_symlink_item_t);
-        last_item->next = item;
-        item->next = NULL;
-        item->key = key;
-        item->val = val;
-        item->hash_value = hash_value;
-        item->secondary_hash_value = secondary_hash_value;
-    }
+    item_placeholder = &(last_item->next);
 
-rehash_check:
+alloc_item:
+    /* Put the new element at the end of the list */
+    item = *(item_placeholder) = fcs_compact_alloc_ptr(&(hash->allocator), sizeof(*item));
+    item->next = NULL;
+    item->key = key;
+    item->val = val;
+    item->hash_value = hash_value;
+#ifdef FCS_ENABLE_SECONDARY_HASH_VALUE
+    item->secondary_hash_value = secondary_hash_value;
+#endif
 
-    hash->num_elems++;
-
-    if (hash->num_elems > ((hash->size*3)>>2))
+    if ((++hash->num_elems) > ((hash->size*3)>>2))
     {
-        SFO_hash_rehash(hash);
+        fc_solve_hash_rehash(hash);
     }
 
     *existing_key = NULL;
@@ -182,75 +178,27 @@ rehash_check:
     return 0;
 }
 
-void fc_solve_hash_free_with_callback(
-    SFO_hash_t * hash,
-    void (*function_ptr)(void * key, void * context)
-    )
-{
-    int i;
-    SFO_hash_symlink_item_t * item, * next_item;
-
-    for(i=0;i<hash->size;i++)
-    {
-        item = hash->entries[i].first_item;
-        while (item != NULL)
-        {
-            function_ptr(item->key, hash->context);
-            next_item = item->next;
-
-            item = next_item;
-        }
-    }
-
-    fc_solve_hash_free(hash);
-}
-
-void fc_solve_hash_free(
-    SFO_hash_t * hash
-    )
-{
-    fc_solve_compact_allocator_finish(hash->allocator);
-
-    free(hash->entries);
-
-    free(hash);
-}
-
-
 /*
     This function "rehashes" a hash. I.e: it increases the size of its
     hash table, allowing for smaller chains, and faster lookup.
 
   */
-static void SFO_hash_rehash(
-    SFO_hash_t * hash
+static void GCC_INLINE fc_solve_hash_rehash(
+    fc_solve_hash_t * hash
     )
 {
     int old_size, new_size, new_size_bitmask;
     int i;
-#if 0
-    SFO_hash_t * new_hash;
-#endif
-    SFO_hash_symlink_item_t * item, * next_item;
+    fc_solve_hash_symlink_item_t * item, * next_item;
     int place;
-    SFO_hash_symlink_t * new_entries;
+    fc_solve_hash_symlink_t * new_entries;
 
     old_size = hash->size;
 
-#if 0
-    /* Allocate a new hash with hash_init() */
-    new_hash = fc_solve_hash_init_proto(
-        old_size * 2,
-        hash->compare_function,
-        hash->context
-        );
-#endif
-
-    old_size = hash->size;
     new_size = old_size << 1;
     new_size_bitmask = new_size - 1;
 
-    new_entries = calloc(new_size, sizeof(SFO_hash_symlink_t));
+    new_entries = calloc(new_size, sizeof(fc_solve_hash_symlink_t));
 
     /* Copy the items to the new hash while not allocating them again */
     for(i=0;i<old_size;i++)
@@ -281,9 +229,6 @@ static void SFO_hash_rehash(
     free(hash->entries);
 
     /* Copy the new hash to the old one */
-#if 0
-    *hash = *new_hash;
-#endif
     hash->entries = new_entries;
     hash->size = new_size;
     hash->size_bitmask = new_size_bitmask;
