@@ -1,33 +1,52 @@
+/* Copyright (c) 2000 Shlomi Fish
+ *
+ * Permission is hereby granted, free of charge, to any person
+ * obtaining a copy of this software and associated documentation
+ * files (the "Software"), to deal in the Software without
+ * restriction, including without limitation the rights to use,
+ * copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following
+ * conditions:
+ *
+ * The above copyright notice and this permission notice shall be
+ * included in all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
+ * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+ * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
+ * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
+ * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+ * OTHER DEALINGS IN THE SOFTWARE.
+ */
 /*
  * lib.c - library interface functions of Freecell Solver.
  *
- * Written by Shlomi Fish ( http://www.shlomifish.org/ ), 2000
- *
- * This file is in the public domain (it's uncopyrighted).
  */
+#define BUILDING_DLL 1
+
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
 
 #include "card.h"
-#include "fcs.h"
+#include "instance.h"
 #include "preset.h"
 #include "fcs_user.h"
+#include "move_funcs_order.h"
 
-#ifdef DMALLOC
-#include "dmalloc.h"
-#endif
+#include "unused.h"
 
-struct fcs_instance_item_struct
+typedef struct 
 {
     fc_solve_instance_t * instance;
     int ret;
     int limit;
-};
+}  fcs_instance_item_t;
 
-typedef struct fcs_instance_item_struct fcs_instance_item_t;
-
-struct fcs_user_struct
+typedef struct
 {
     /*
      * This is a list of several consecutive instances that are run
@@ -36,7 +55,6 @@ struct fcs_user_struct
      * */
     fcs_instance_item_t * instances_list;
     int num_instances;
-    int max_num_instances;
 
     int current_instance_idx;
     /*
@@ -71,15 +89,16 @@ struct fcs_user_struct
 #endif
     char * state_string_copy;
 
+#ifndef FCS_FREECELL_ONLY
     fcs_preset_t common_preset;
-};
-
-typedef struct fcs_user_struct fcs_user_t;
+#endif
+} fcs_user_t;
 
 static void user_initialize(
         fcs_user_t * ret
         )
 {
+#ifndef FCS_FREECELL_ONLY
     const fcs_preset_t * freecell_preset;
 
     fc_solve_get_preset_by_name(
@@ -88,28 +107,28 @@ static void user_initialize(
         );
 
     fcs_duplicate_preset(ret->common_preset, *freecell_preset);
+#endif
 
-    ret->max_num_instances = 10;
-    ret->instances_list = malloc(sizeof(ret->instances_list[0]) * ret->max_num_instances);
+    ret->instances_list = malloc(sizeof(ret->instances_list[0]));
     ret->num_instances = 1;
     ret->current_instance_idx = 0;
     ret->instance = fc_solve_alloc_instance();
+#ifndef FCS_FREECELL_ONLY
     fc_solve_apply_preset_by_ptr(ret->instance, &(ret->common_preset));
+#endif
     ret->instances_list[ret->current_instance_idx].instance = ret->instance;
     ret->instances_list[ret->current_instance_idx].ret = ret->ret = FCS_STATE_NOT_BEGAN_YET;
     ret->instances_list[ret->current_instance_idx].limit = -1;
     ret->current_iterations_limit = -1;
 
     ret->soft_thread =
-        fc_solve_instance_get_soft_thread(
-            ret->instance, 0,0
-            );
+        fc_solve_instance_get_first_soft_thread(ret->instance);
 
     ret->state_string_copy = NULL;
     ret->iterations_board_started_at = 0;
 }
 
-void * freecell_solver_user_alloc(void)
+void DLLEXPORT * freecell_solver_user_alloc(void)
 {
     fcs_user_t * ret;
 
@@ -120,10 +139,13 @@ void * freecell_solver_user_alloc(void)
     return (void*)ret;
 }
 
-int freecell_solver_user_apply_preset(
+int DLLEXPORT freecell_solver_user_apply_preset(
     void * user_instance,
     const char * preset_name)
 {
+#ifdef FCS_FREECELL_ONLY
+    return FCS_PRESET_CODE_OK;
+#else
     const fcs_preset_t * new_preset_ptr;
     fcs_user_t * user;
     int status;
@@ -158,9 +180,10 @@ int freecell_solver_user_apply_preset(
     fcs_duplicate_preset(user->common_preset, *new_preset_ptr);
 
     return FCS_PRESET_CODE_OK;
+#endif
 }
 
-void freecell_solver_user_limit_iterations(
+void DLLEXPORT freecell_solver_user_limit_iterations(
     void * user_instance,
     int max_iters
     )
@@ -172,7 +195,7 @@ void freecell_solver_user_limit_iterations(
     user->current_iterations_limit = max_iters;
 }
 
-void freecell_solver_user_limit_current_instance_iterations(
+void DLLEXPORT freecell_solver_user_limit_current_instance_iterations(
     void * user_instance,
     int max_iters
     )
@@ -188,7 +211,7 @@ void freecell_solver_user_limit_current_instance_iterations(
 #define min(a,b) (((a)<(b))?(a):(b))
 #endif
 
-int freecell_solver_user_set_tests_order(
+int DLLEXPORT freecell_solver_user_set_tests_order(
     void * user_instance,
     const char * tests_order,
     char * * error_string
@@ -206,7 +229,7 @@ int freecell_solver_user_set_tests_order(
             );
 }
 
-int freecell_solver_user_solve_board(
+int DLLEXPORT freecell_solver_user_solve_board(
     void * user_instance,
     const char * state_as_string
     )
@@ -229,13 +252,16 @@ static void recycle_instance(
 {
     if (user->instances_list[i].ret == FCS_STATE_WAS_SOLVED)
     {
-        fcs_move_stack_destroy(user->instance->solution_moves);
-        user->instance->solution_moves = NULL;
+        fcs_move_stack_static_destroy(user->instance->solution_moves);
+        user->instance->solution_moves.moves = NULL;
     }
+    /* fc_solve_unresume_instance is empty. */
+#if 0
     else if (user->instances_list[i].ret == FCS_STATE_SUSPEND_PROCESS)
     {
         fc_solve_unresume_instance(user->instances_list[i].instance);
     }
+#endif
 
     if (user->instances_list[i].ret != FCS_STATE_NOT_BEGAN_YET)
     {
@@ -251,7 +277,7 @@ static void recycle_instance(
     user->instances_list[i].ret = FCS_STATE_NOT_BEGAN_YET;
 }
 
-int freecell_solver_user_resume_solution(
+int DLLEXPORT freecell_solver_user_resume_solution(
     void * user_instance
     )
 {
@@ -262,6 +288,8 @@ int freecell_solver_user_resume_solution(
 
     user = (fcs_user_t*)user_instance;
 
+    ret = FCS_STATE_IS_NOT_SOLVEABLE;
+
     /*
      * I expect user->current_instance_idx to be initialized at some value.
      * */
@@ -277,7 +305,9 @@ int freecell_solver_user_resume_solution(
         if (user->instances_list[user->current_instance_idx].ret == FCS_STATE_NOT_BEGAN_YET)
         {
             int status;
+#if (!(defined(HARD_CODED_NUM_FREECELLS) && defined(HARD_CODED_NUM_STACKS) && defined(HARD_CODED_NUM_DECKS)))
             fc_solve_instance_t * instance = user->instance;
+#endif
 
             status = fc_solve_initial_user_state_to_c(
                 user->state_string_copy,
@@ -311,7 +341,7 @@ int freecell_solver_user_resume_solution(
 #endif
                 &(user->state_validity_card));
 
-            if (user->state_validity_ret != 0)
+            if (user->state_validity_ret != FCS_STATE_VALIDITY__OK)
             {
                 user->ret = FCS_STATE_INVALID_STATE;
                 return user->ret;
@@ -322,10 +352,10 @@ int freecell_solver_user_resume_solution(
              * state to it before state is canonized
              * */
             fcs_duplicate_state(
-                user->running_state.s,
-                user->running_state.info,
-                user->state.s,
-                user->state.info
+                &(user->running_state.s),
+                &(user->running_state.info),
+                &(user->state.s),
+                &(user->state.info)
                 );
 
             fc_solve_canonize_state(
@@ -398,9 +428,11 @@ int freecell_solver_user_resume_solution(
 
         if (user->ret == FCS_STATE_WAS_SOLVED)
         {
+#if (!(defined(HARD_CODED_NUM_FREECELLS) && defined(HARD_CODED_NUM_STACKS) && defined(HARD_CODED_NUM_DECKS)))
             fc_solve_instance_t * instance = user->instance;
+#endif
             fc_solve_move_stack_normalize(
-                user->instance->solution_moves,
+                &(user->instance->solution_moves),
                 &(user->state.info),
                 INSTANCE_FREECELLS_NUM,
                 INSTANCE_STACKS_NUM,
@@ -437,7 +469,7 @@ int freecell_solver_user_resume_solution(
     return ret;
 }
 
-int freecell_solver_user_get_next_move(
+int DLLEXPORT freecell_solver_user_get_next_move(
     void * user_instance,
     fcs_move_t * move
     )
@@ -447,13 +479,15 @@ int freecell_solver_user_get_next_move(
     user = (fcs_user_t*)user_instance;
 
     {
+#if (!(defined(HARD_CODED_NUM_FREECELLS) && defined(HARD_CODED_NUM_STACKS) && defined(HARD_CODED_NUM_DECKS)))
         fc_solve_instance_t * instance = user->instance;
+#endif
         if (user->ret == FCS_STATE_WAS_SOLVED)
         {
             int ret;
 
-            ret = fcs_move_stack_pop(
-                user->instance->solution_moves,
+            ret = fc_solve_move_stack_pop(
+                &(user->instance->solution_moves),
                 move
                 );
 
@@ -476,7 +510,7 @@ int freecell_solver_user_get_next_move(
     }
 }
 
-char * freecell_solver_user_current_state_as_string(
+DLLEXPORT char * freecell_solver_user_current_state_as_string(
     void * user_instance,
     int parseable_output,
     int canonized_order_output,
@@ -488,7 +522,9 @@ char * freecell_solver_user_current_state_as_string(
     user = (fcs_user_t *)user_instance;
 
     {
+#if (!(defined(HARD_CODED_NUM_FREECELLS) && defined(HARD_CODED_NUM_STACKS) && defined(HARD_CODED_NUM_DECKS)))
         fc_solve_instance_t * instance = user->instance;
+#endif
 
         return
             fc_solve_state_as_string(
@@ -515,13 +551,16 @@ static void user_free_resources(
 
         if (ret_code == FCS_STATE_WAS_SOLVED)
         {
-            fcs_move_stack_destroy(user->instance->solution_moves);
-            user->instance->solution_moves = NULL;
+            fcs_move_stack_static_destroy(user->instance->solution_moves);
+            user->instance->solution_moves.moves = NULL;
         }
+        /* fc_solve_unresume_instance is empty. */
+#if 0
         else if (ret_code == FCS_STATE_SUSPEND_PROCESS)
         {
             fc_solve_unresume_instance(user->instances_list[i].instance);
         }
+#endif
 
         if (ret_code != FCS_STATE_NOT_BEGAN_YET)
         {
@@ -543,7 +582,7 @@ static void user_free_resources(
     }
 }
 
-void freecell_solver_user_free(
+void DLLEXPORT freecell_solver_user_free(
     void * user_instance
     )
 {
@@ -556,7 +595,7 @@ void freecell_solver_user_free(
     free(user);
 }
 
-int freecell_solver_user_get_current_depth(
+int DLLEXPORT freecell_solver_user_get_current_depth(
     void * user_instance
     )
 {
@@ -564,10 +603,10 @@ int freecell_solver_user_get_current_depth(
 
     user = (fcs_user_t *)user_instance;
 
-    return (user->soft_thread->num_solution_states - 1);
+    return (user->soft_thread->method_specific.soft_dfs.depth);
 }
 
-void freecell_solver_user_set_solving_method(
+void DLLEXPORT freecell_solver_user_set_solving_method(
     void * user_instance,
     int method
     )
@@ -577,6 +616,35 @@ void freecell_solver_user_set_solving_method(
     user = (fcs_user_t *)user_instance;
 
     user->soft_thread->method = method;
+
+    switch (method)
+    {
+        case FCS_METHOD_A_STAR:
+        {
+            double * my_a_star_weights;
+            int a;
+
+            my_a_star_weights = user->soft_thread->method_specific.befs.meth.befs.a_star_weights;
+            for(a = 0;
+                a<(sizeof(fc_solve_a_star_default_weights)/
+                    sizeof(fc_solve_a_star_default_weights[0]));
+                a++)
+            {
+                my_a_star_weights[a] = fc_solve_a_star_default_weights[a];
+            }
+
+            user->soft_thread->method_specific.befs.meth.befs.a_star_pqueue.Elements = NULL;
+        }
+        break;
+        case FCS_METHOD_OPTIMIZE:
+        case FCS_METHOD_BFS:
+        {
+            user->soft_thread->method_specific.befs.meth.brfs.bfs_queue =
+            user->soft_thread->method_specific.befs.meth.brfs.bfs_queue_last_item =
+            NULL;
+        }
+        break;
+    }
 }
 
 #define set_for_all_instances(what) \
@@ -588,15 +656,14 @@ void freecell_solver_user_set_solving_method(
     user->common_preset.what = what;     \
     }
 
-int freecell_solver_user_set_num_freecells(
+#ifndef HARD_CODED_NUM_FREECELLS
+int DLLEXPORT freecell_solver_user_set_num_freecells(
     void * user_instance,
     int freecells_num
     )
 {
     fcs_user_t * user;
-#ifndef HARD_CODED_NUM_FREECELLS
     int i;
-#endif
 
     user = (fcs_user_t *)user_instance;
 
@@ -605,22 +672,28 @@ int freecell_solver_user_set_num_freecells(
         return 1;
     }
 
-#ifndef HARD_CODED_NUM_FREECELLS
     set_for_all_instances(freecells_num);
-#endif
 
     return 0;
 }
+#else
+int DLLEXPORT freecell_solver_user_set_num_freecells(
+    void * user_instance GCC_UNUSED,
+    int freecells_num GCC_UNUSED
+    )
+{
+    return 0;
+}
+#endif
 
-int freecell_solver_user_set_num_stacks(
+#ifndef HARD_CODED_NUM_STACKS
+int DLLEXPORT freecell_solver_user_set_num_stacks(
     void * user_instance,
     int stacks_num
     )
 {
     fcs_user_t * user;
-#ifndef HARD_CODED_NUM_STACKS
     int i;
-#endif
 
     user = (fcs_user_t *)user_instance;
 
@@ -628,22 +701,28 @@ int freecell_solver_user_set_num_stacks(
     {
         return 1;
     }
-#ifndef HARD_CODED_NUM_STACKS
     set_for_all_instances(stacks_num);
-#endif
 
     return 0;
 }
+#else
+int DLLEXPORT freecell_solver_user_set_num_stacks(
+    void * user_instance GCC_UNUSED,
+    int stacks_num GCC_UNUSED
+    )
+{
+    return 0;
+}
+#endif
 
-int freecell_solver_user_set_num_decks(
+#ifndef HARD_CODED_NUM_DECKS
+int DLLEXPORT freecell_solver_user_set_num_decks(
     void * user_instance,
     int decks_num
     )
 {
     fcs_user_t * user;
-#ifndef HARD_CODED_NUM_DECKS
     int i;
-#endif
 
     user = (fcs_user_t *)user_instance;
 
@@ -651,15 +730,21 @@ int freecell_solver_user_set_num_decks(
     {
         return 1;
     }
-#ifndef HARD_CODED_NUM_DECKS
     set_for_all_instances(decks_num);
-#endif
 
     return 0;
 }
+#else
+int DLLEXPORT freecell_solver_user_set_num_decks(
+    void * user_instance GCC_UNUSED,
+    int decks_num GCC_UNUSED
+    )
+{
+    return 0;
+}
+#endif
 
-
-int freecell_solver_user_set_game(
+int DLLEXPORT freecell_solver_user_set_game(
     void * user_instance,
     int freecells_num,
     int stacks_num,
@@ -669,10 +754,6 @@ int freecell_solver_user_set_game(
     int empty_stacks_fill
     )
 {
-    fcs_user_t * user;
-
-    user = (fcs_user_t *)user_instance;
-
     if (freecell_solver_user_set_num_freecells(user_instance, freecells_num))
     {
         return 1;
@@ -701,7 +782,7 @@ int freecell_solver_user_set_game(
     return 0;
 }
 
-int freecell_solver_user_get_num_times(void * user_instance)
+int DLLEXPORT freecell_solver_user_get_num_times(void * user_instance)
 {
     fcs_user_t * user;
 
@@ -710,7 +791,7 @@ int freecell_solver_user_get_num_times(void * user_instance)
     return user->iterations_board_started_at + user->instance->num_times - user->init_num_times;
 }
 
-int freecell_solver_user_get_limit_iterations(void * user_instance)
+int DLLEXPORT freecell_solver_user_get_limit_iterations(void * user_instance)
 {
     fcs_user_t * user;
 
@@ -719,18 +800,18 @@ int freecell_solver_user_get_limit_iterations(void * user_instance)
     return user->instance->max_num_times;
 }
 
-int freecell_solver_user_get_moves_left(void * user_instance)
+int DLLEXPORT freecell_solver_user_get_moves_left(void * user_instance)
 {
     fcs_user_t * user;
 
     user = (fcs_user_t *)user_instance;
     if (user->ret == FCS_STATE_WAS_SOLVED)
-        return user->instance->solution_moves->num_moves;
+        return user->instance->solution_moves.num_moves;
     else
         return 0;
 }
 
-void freecell_solver_user_set_solution_optimization(
+void DLLEXPORT freecell_solver_user_set_solution_optimization(
     void * user_instance,
     int optimize
 )
@@ -742,7 +823,7 @@ void freecell_solver_user_set_solution_optimization(
     user->instance->optimize_solution_path = optimize;
 }
 
-char * freecell_solver_user_move_to_string(
+DLLEXPORT char * freecell_solver_user_move_to_string(
     fcs_move_t move,
     int standard_notation
     )
@@ -750,17 +831,21 @@ char * freecell_solver_user_move_to_string(
     return fc_solve_move_to_string(move, standard_notation);
 }
 
-char * freecell_solver_user_move_to_string_w_state(
+DLLEXPORT char * freecell_solver_user_move_to_string_w_state(
     void * user_instance,
     fcs_move_t move,
     int standard_notation
     )
 {
     fcs_user_t * user;
+#if (!(defined(HARD_CODED_NUM_FREECELLS) && defined(HARD_CODED_NUM_STACKS) && defined(HARD_CODED_NUM_DECKS)))
     fc_solve_instance_t * instance;
+#endif
 
     user = (fcs_user_t *)user_instance;
+#if (!(defined(HARD_CODED_NUM_FREECELLS) && defined(HARD_CODED_NUM_STACKS) && defined(HARD_CODED_NUM_DECKS)))    
     instance = user->instance;
+#endif
 
     return
         fc_solve_move_to_string_w_state(
@@ -773,7 +858,7 @@ char * freecell_solver_user_move_to_string_w_state(
             );
 }
 
-void freecell_solver_user_limit_depth(
+void DLLEXPORT freecell_solver_user_limit_depth(
     void * user_instance,
     int max_depth
 )
@@ -785,17 +870,17 @@ void freecell_solver_user_limit_depth(
     user->instance->max_depth = max_depth;
 }
 
-int freecell_solver_user_get_max_num_freecells(void)
+int DLLEXPORT freecell_solver_user_get_max_num_freecells(void)
 {
     return MAX_NUM_FREECELLS;
 }
 
-int freecell_solver_user_get_max_num_stacks(void)
+int DLLEXPORT freecell_solver_user_get_max_num_stacks(void)
 {
     return MAX_NUM_STACKS;
 }
 
-int freecell_solver_user_get_max_num_decks(void)
+int DLLEXPORT freecell_solver_user_get_max_num_decks(void)
 {
     return MAX_NUM_DECKS;
 }
@@ -815,7 +900,7 @@ char * freecell_solver_user_get_invalid_state_error_string(
     {
         return strdup("");
     }
-    fcs_card_perl2user(user->state_validity_card, card_str, print_ts);
+    fc_solve_card_perl2user(user->state_validity_card, card_str, print_ts);
 
     if (user->state_validity_ret == FCS_STATE_VALIDITY__EMPTY_SLOT)
     {
@@ -839,11 +924,12 @@ char * freecell_solver_user_get_invalid_state_error_string(
     return strdup(string);
 }
 
-int freecell_solver_user_set_sequences_are_built_by_type(
+int DLLEXPORT freecell_solver_user_set_sequences_are_built_by_type(
     void * user_instance,
     int sequences_are_built_by
     )
 {
+#ifndef FCS_FREECELL_ONLY
     fcs_user_t * user;
     int i;
 
@@ -854,30 +940,33 @@ int freecell_solver_user_set_sequences_are_built_by_type(
         return 1;
     }
     set_for_all_instances(sequences_are_built_by)
+#endif
 
     return 0;
 }
 
-int freecell_solver_user_set_sequence_move(
+int DLLEXPORT freecell_solver_user_set_sequence_move(
     void * user_instance,
     int unlimited_sequence_move
     )
 {
+#ifndef FCS_FREECELL_ONLY
     fcs_user_t * user;
     int i;
 
     user = (fcs_user_t *)user_instance;
 
     set_for_all_instances(unlimited_sequence_move);
-
+#endif
     return 0;
 }
 
-int freecell_solver_user_set_empty_stacks_filled_by(
+int DLLEXPORT freecell_solver_user_set_empty_stacks_filled_by(
     void * user_instance,
     int empty_stacks_fill
     )
 {
+#ifndef FCS_FREECELL_ONLY
     fcs_user_t * user;
     int i;
 
@@ -888,11 +977,11 @@ int freecell_solver_user_set_empty_stacks_filled_by(
         return 1;
     }
     set_for_all_instances(empty_stacks_fill);
-
+#endif
     return 0;
 }
 
-int freecell_solver_user_set_a_star_weight(
+int DLLEXPORT freecell_solver_user_set_a_star_weight(
     void * user_instance,
     int index,
     double weight
@@ -902,7 +991,8 @@ int freecell_solver_user_set_a_star_weight(
 
     user = (fcs_user_t *)user_instance;
 
-    if ((index < 0) || (index >= (sizeof(user->soft_thread->a_star_weights)/sizeof(user->soft_thread->a_star_weights[0]))))
+#define my_a_star_weights soft_thread->method_specific.befs.meth.befs.a_star_weights
+    if ((index < 0) || (index >= (sizeof(user->my_a_star_weights)/sizeof(user->my_a_star_weights[0]))))
     {
         return 1;
     }
@@ -911,7 +1001,7 @@ int freecell_solver_user_set_a_star_weight(
         return 2;
     }
 
-    user->soft_thread->a_star_weights[index] = weight;
+    user->my_a_star_weights[index] = weight;
 
     return 0;
 
@@ -921,7 +1011,7 @@ static void iter_handler_wrapper(
     void * user_instance,
     int iter_num,
     int depth,
-    void * lp_instance,
+    void * lp_instance GCC_UNUSED,
     fcs_state_extra_info_t * ptr_state_val,
     int parent_iter_num
     )
@@ -946,7 +1036,7 @@ static void iter_handler_wrapper(
     return;
 }
 
-void freecell_solver_user_set_iter_handler(
+void DLLEXPORT freecell_solver_user_set_iter_handler(
     void * user_instance,
     freecell_solver_user_iter_handler_t iter_handler,
     void * iter_handler_context
@@ -972,20 +1062,30 @@ void freecell_solver_user_set_iter_handler(
     }
 }
 
-char * freecell_solver_user_iter_state_as_string(
-    void * user_instance,
+#if (!(defined(HARD_CODED_NUM_FREECELLS) && defined(HARD_CODED_NUM_STACKS) && defined(HARD_CODED_NUM_DECKS)))
+#define HARD_CODED_UNUSED 
+#else
+#define HARD_CODED_UNUSED GCC_UNUSED
+#endif
+
+DLLEXPORT char * freecell_solver_user_iter_state_as_string(
+    void * user_instance HARD_CODED_UNUSED,
     void * ptr_state_void,
     int parseable_output,
     int canonized_order_output,
     int display_10_as_t
 )
 {
+#if (!(defined(HARD_CODED_NUM_FREECELLS) && defined(HARD_CODED_NUM_STACKS) && defined(HARD_CODED_NUM_DECKS)))
     fcs_user_t * user;
     fc_solve_instance_t * instance;
+#endif
     fcs_standalone_state_ptrs_t * ptr_state;
 
+#if (!(defined(HARD_CODED_NUM_FREECELLS) && defined(HARD_CODED_NUM_STACKS) && defined(HARD_CODED_NUM_DECKS)))
     user = (fcs_user_t *)user_instance;
     instance = user->instance;
+#endif
 
     ptr_state = (fcs_standalone_state_ptrs_t *)ptr_state_void;
 
@@ -1001,7 +1101,7 @@ char * freecell_solver_user_iter_state_as_string(
             );
 }
 
-void freecell_solver_user_set_random_seed(
+void DLLEXPORT freecell_solver_user_set_random_seed(
     void * user_instance,
     int seed
 )
@@ -1010,10 +1110,13 @@ void freecell_solver_user_set_random_seed(
 
     user = (fcs_user_t *)user_instance;
 
-    fc_solve_rand_srand(user->soft_thread->rand_gen, (user->soft_thread->rand_seed = seed));
+    fc_solve_rand_init(
+            &(user->soft_thread->method_specific.soft_dfs.rand_gen), 
+            (user->soft_thread->method_specific.soft_dfs.rand_seed = seed)
+            );
 }
 
-int freecell_solver_user_get_num_states_in_collection(void * user_instance)
+int DLLEXPORT freecell_solver_user_get_num_states_in_collection(void * user_instance)
 {
     fcs_user_t * user;
 
@@ -1022,7 +1125,7 @@ int freecell_solver_user_get_num_states_in_collection(void * user_instance)
     return user->instance->num_states_in_collection;
 }
 
-void freecell_solver_user_limit_num_states_in_collection(
+void DLLEXPORT freecell_solver_user_limit_num_states_in_collection(
     void * user_instance,
     int max_num_states
     )
@@ -1034,7 +1137,7 @@ void freecell_solver_user_limit_num_states_in_collection(
     user->instance->max_num_states_in_collection = max_num_states;
 }
 
-int freecell_solver_user_next_soft_thread(
+int DLLEXPORT freecell_solver_user_next_soft_thread(
     void * user_instance
     )
 {
@@ -1043,7 +1146,7 @@ int freecell_solver_user_next_soft_thread(
 
     user = (fcs_user_t *)user_instance;
 
-    soft_thread = fc_solve_new_soft_thread(user->soft_thread);
+    soft_thread = fc_solve_new_soft_thread(user->soft_thread->hard_thread);
 
     if (soft_thread == NULL)
     {
@@ -1055,7 +1158,7 @@ int freecell_solver_user_next_soft_thread(
     return 0;
 }
 
-extern void freecell_solver_user_set_soft_thread_step(
+extern void DLLEXPORT freecell_solver_user_set_soft_thread_step(
     void * user_instance,
     int num_times_step
     )
@@ -1067,7 +1170,7 @@ extern void freecell_solver_user_set_soft_thread_step(
     user->soft_thread->num_times_step = num_times_step;
 }
 
-int freecell_solver_user_next_hard_thread(
+int DLLEXPORT freecell_solver_user_next_hard_thread(
     void * user_instance
     )
 {
@@ -1088,7 +1191,7 @@ int freecell_solver_user_next_hard_thread(
     return 0;
 }
 
-int freecell_solver_user_get_num_soft_threads_in_instance(
+int DLLEXPORT freecell_solver_user_get_num_soft_threads_in_instance(
     void * user_instance
     )
 {
@@ -1099,7 +1202,7 @@ int freecell_solver_user_get_num_soft_threads_in_instance(
     return user->instance->next_soft_thread_id;
 }
 
-void freecell_solver_user_set_calc_real_depth(
+void DLLEXPORT freecell_solver_user_set_calc_real_depth(
     void * user_instance,
     int calc_real_depth
 )
@@ -1111,7 +1214,7 @@ void freecell_solver_user_set_calc_real_depth(
     user->instance->calc_real_depth = calc_real_depth;
 }
 
-void freecell_solver_user_set_soft_thread_name(
+void DLLEXPORT freecell_solver_user_set_soft_thread_name(
     void * user_instance,
     char * name
     )
@@ -1127,7 +1230,7 @@ void freecell_solver_user_set_soft_thread_name(
     user->soft_thread->name = strdup(name);
 }
 
-int freecell_solver_user_set_hard_thread_prelude(
+int DLLEXPORT freecell_solver_user_set_hard_thread_prelude(
     void * user_instance,
     char * prelude
     )
@@ -1149,7 +1252,7 @@ int freecell_solver_user_set_hard_thread_prelude(
     return 0;
 }
 
-void freecell_solver_user_recycle(
+void DLLEXPORT freecell_solver_user_recycle(
     void * user_instance
     )
 {
@@ -1171,7 +1274,7 @@ void freecell_solver_user_recycle(
     }
 }
 
-int freecell_solver_user_set_optimization_scan_tests_order(
+int DLLEXPORT freecell_solver_user_set_optimization_scan_tests_order(
     void * user_instance,
     const char * tests_order,
     char * * error_string
@@ -1205,7 +1308,7 @@ int freecell_solver_user_set_optimization_scan_tests_order(
     return ret;
 }
 
-void freecell_solver_user_set_reparent_states(
+void DLLEXPORT freecell_solver_user_set_reparent_states(
     void * user_instance,
     int to_reparent_states
     )
@@ -1214,10 +1317,10 @@ void freecell_solver_user_set_reparent_states(
 
     user = (fcs_user_t *)user_instance;
 
-    user->instance->to_reparent_states = to_reparent_states;
+    user->instance->to_reparent_states_proto = to_reparent_states;
 }
 
-void freecell_solver_user_set_scans_synergy(
+void DLLEXPORT freecell_solver_user_set_scans_synergy(
     void * user_instance,
     int synergy
     )
@@ -1229,7 +1332,7 @@ void freecell_solver_user_set_scans_synergy(
     user->instance->scans_synergy = synergy;
 }
 
-int freecell_solver_user_next_instance(
+int DLLEXPORT freecell_solver_user_next_instance(
     void * user_instance
     )
 {
@@ -1237,29 +1340,26 @@ int freecell_solver_user_next_instance(
 
     user = (fcs_user_t *)user_instance;
 
-    user->num_instances++;
-    if (user->num_instances == user->max_num_instances)
-    {
-        user->max_num_instances += 10;
-        user->instances_list =
-            realloc(
-                user->instances_list,
-                sizeof(user->instances_list[0])*user->max_num_instances
-                );
-    }
+    user->instances_list =
+        realloc(
+            user->instances_list,
+            sizeof(user->instances_list[0])
+            * (++user->num_instances)
+            );
+
     user->current_instance_idx = user->num_instances-1;
     user->instance = fc_solve_alloc_instance();
 
+#ifndef FCS_FREECELL_ONLY
     fc_solve_apply_preset_by_ptr(user->instance, &(user->common_preset));
+#endif
 
     /*
      * Switch the soft_thread variable so it won't refer to the old
      * instance
      * */
     user->soft_thread =
-        fc_solve_instance_get_soft_thread(
-            user->instance, 0, 0
-            );
+        fc_solve_instance_get_first_soft_thread(user->instance);
 
     user->instances_list[user->current_instance_idx].instance = user->instance;
     user->instances_list[user->current_instance_idx].ret = user->ret = FCS_STATE_NOT_BEGAN_YET;
@@ -1268,7 +1368,7 @@ int freecell_solver_user_next_instance(
     return 0;
 }
 
-int freecell_solver_user_reset(void * user_instance)
+int DLLEXPORT freecell_solver_user_reset(void * user_instance)
 {
     fcs_user_t * user;
 
@@ -1281,7 +1381,9 @@ int freecell_solver_user_reset(void * user_instance)
     return 0;
 }
 
-const char * freecell_solver_user_get_lib_version(void * user_instance)
+DLLEXPORT const char * freecell_solver_user_get_lib_version(
+    void * user_instance GCC_UNUSED
+    )
 {
     return VERSION;
 }
