@@ -1,9 +1,40 @@
+/* Copyright (c) 2000 Shlomi Fish
+ *
+ * Permission is hereby granted, free of charge, to any person
+ * obtaining a copy of this software and associated documentation
+ * files (the "Software"), to deal in the Software without
+ * restriction, including without limitation the rights to use,
+ * copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following
+ * conditions:
+ *
+ * The above copyright notice and this permission notice shall be
+ * included in all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
+ * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+ * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
+ * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
+ * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+ * OTHER DEALINGS IN THE SOFTWARE.
+ */
+/*
+ * main.c - the main() and related functions of the fc-solve command line
+ * executable.
+ *
+ * It is documented in the documents "README", "USAGE", etc. in the 
+ * Freecell Solver distribution from http://fc-solve.berlios.de/ .
+ */
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <signal.h>
 
 #include "fcs_cl.h"
+#include "unused.h"
 
 struct fc_solve_display_information_context_struct
 {
@@ -76,51 +107,13 @@ static void my_iter_handler(
         printf("%s\n---------------\n\n\n", state_string);
         free((void*)state_string);
     }
-
-#ifdef MYDEBUG
-    {
-        fcs_card_t card;
-        int ret;
-        char card_str[10];
-
-        ret = fcs_check_state_validity(
-            ptr_state_with_locations,
-            context->freecells_num,
-            context->stacks_num,
-            context->decks_num,
-            &card
-            );
-
-        if (ret != 0)
-        {
-
-            fcs_card_perl2user(card, card_str, context->display_10_as_t);
-            if (ret == 3)
-            {
-                fprintf(stdout, "%s\n",
-                    "There's an empty slot in one of the stacks."
-                    );
-            }
-            else
-            {
-                fprintf(stdout, "%s%s.\n",
-                    ((ret == 2)? "There's an extra card: " : "There's a missing card: "),
-                    card_str
-                );
-            }
-            exit(-1);
-        }
-    }
-#endif
 }
 
-struct help_screen_struct
+typedef struct
 {
-    char * key;
-    char * screen;
-};
-
-typedef struct help_screen_struct help_screen_t;
+    const char * key;
+    const char * screen;
+} help_screen_t;
 
 help_screen_t help_screens[] = {
 {
@@ -241,8 +234,8 @@ help_screen_t help_screens[] = {
 "        'A' - move a stack card to an empty stack.\n"
 "        'B' - move a stack card to a parent on a different stack.\n"
 "        'C' - move a stack card to a freecell.\n"
-"        'D' - move a freecel card to a parent.\n"
-"        'E' - move a freecel card to an empty stack.\n"
+"        'D' - move a freecell card to a parent.\n"
+"        'E' - move a freecell card to an empty stack.\n"
 "\n"
 "        Simple Simon Tests:\n"
 "\n"
@@ -257,6 +250,7 @@ help_screen_t help_screens[] = {
 "        'g' - move a whole stack sequence to a false parent which has some\n"
 "              cards above it.\n"
 "        'h' - move a sequence to a parent on the same stack.\n"
+"        'i' - move a sequence to a false parent.\n"
 "\n"
 "        Tests are grouped with parenthesis or square brackets. Each group\n"
 "        will be randomized as a whole by the random-dfs scan.\n"
@@ -400,10 +394,9 @@ help_screen_t help_screens[] = {
 }
 ;
 
-enum MY_FCS_CMD_LINE_RET_VALUES
+enum
 {
-    EXIT_AND_RETURN_0 = FCS_CMD_LINE_USER,
-
+    EXIT_AND_RETURN_0 = FCS_CMD_LINE_USER
 };
 
 static void print_help_string(char * key)
@@ -420,7 +413,7 @@ static void print_help_string(char * key)
 
 static int cmd_line_callback(
     void * instance,
-    int argc,
+    int argc GCC_UNUSED,
     char * argv[],
     int arg,
     int * num_to_skip,
@@ -538,7 +531,7 @@ static int cmd_line_callback(
 static int command_num = 0;
 static int debug_iter_output_on = 0;
 
-static void select_signal_handler(int signal_num)
+static void select_signal_handler(int signal_num GCC_UNUSED)
 {
     command_num = (command_num+1)%3;
 }
@@ -547,7 +540,7 @@ static void * current_instance;
 static fc_solve_display_information_context_t * dc;
 
 
-static void command_signal_handler(int signal_num)
+static void command_signal_handler(int signal_num GCC_UNUSED)
 {
     if (command_num == 0)
     {
@@ -612,7 +605,6 @@ int main(int argc, char * argv[])
 {
     int parser_ret;
     void * instance;
-    char * error_string;
     int arg;
     FILE * file;
     char user_state[USER_STATE_SIZE];
@@ -629,43 +621,46 @@ int main(int argc, char * argv[])
     current_instance = instance;
 
 
-    parser_ret =
-        freecell_solver_user_cmd_line_parse_args(
-            instance,
-            argc,
-            argv,
-            1,
-            known_parameters,
-            cmd_line_callback,
-            &debug_context,
-            &error_string,
-            &arg
-            );
+    {
+        char * error_string;
+        parser_ret =
+            freecell_solver_user_cmd_line_parse_args(
+                instance,
+                argc,
+                argv,
+                1,
+                known_parameters,
+                cmd_line_callback,
+                &debug_context,
+                &error_string,
+                &arg
+                );
 
-    if (parser_ret == EXIT_AND_RETURN_0)
-    {
-        freecell_solver_user_free(instance);
-        return 0;
-    }
-    else if (
-        (parser_ret == FCS_CMD_LINE_PARAM_WITH_NO_ARG)
-            )
-    {
-        fprintf(stderr, "The command line parameter \"%s\" requires an argument"
-                " and was not supplied with one.\n", argv[arg]);
-        return (-1);
-    }
-    else if (
-        (parser_ret == FCS_CMD_LINE_ERROR_IN_ARG)
-        )
-    {
-        if (error_string != NULL)
+        if (parser_ret == EXIT_AND_RETURN_0)
         {
-            fprintf(stderr, "%s", error_string);
-            free(error_string);
+            freecell_solver_user_free(instance);
+            return 0;
+        }
+        else if (
+            (parser_ret == FCS_CMD_LINE_PARAM_WITH_NO_ARG)
+                )
+        {
+            fprintf(stderr, "The command line parameter \"%s\" requires an argument"
+                    " and was not supplied with one.\n", argv[arg]);
+            return (-1);
+        }
+        else if (
+            (parser_ret == FCS_CMD_LINE_ERROR_IN_ARG)
+            )
+        {
+            if (error_string != NULL)
+            {
+                fprintf(stderr, "%s", error_string);
+                free(error_string);
+            }
+            freecell_solver_user_free(instance);
+            return -1;
         }
-        freecell_solver_user_free(instance);
-        return -1;
     }
 
     if ((arg == argc) || (!strcmp(argv[arg], "-")))
@@ -735,6 +730,7 @@ int main(int argc, char * argv[])
     if (ret == FCS_STATE_INVALID_STATE)
     {
         char * error_string;
+
         error_string =
             freecell_solver_user_get_invalid_state_error_string(
                 instance,
@@ -742,6 +738,7 @@ int main(int argc, char * argv[])
                 );
         printf("%s\n", error_string);
         free(error_string);
+        error_string = NULL;
     }
     else
     {
