@@ -1,24 +1,43 @@
+/* Copyright (c) 2000 Shlomi Fish
+ *
+ * Permission is hereby granted, free of charge, to any person
+ * obtaining a copy of this software and associated documentation
+ * files (the "Software"), to deal in the Software without
+ * restriction, including without limitation the rights to use,
+ * copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following
+ * conditions:
+ *
+ * The above copyright notice and this permission notice shall be
+ * included in all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
+ * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+ * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
+ * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
+ * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+ * OTHER DEALINGS IN THE SOFTWARE.
+ */
 /*
  * preset.c - game presets management for Freecell Solver
  *
- * Written by Shlomi Fish ( http://www.shlomifish.org/ ), 2000
- *
- * This file is in the public domain (it's uncopyrighted).
- *
  */
 
+#define BUILDING_DLL 1
 
 #include <string.h>
 #include <stdlib.h>
 
-#include "fcs.h"
+#include "instance.h"
 #include "preset.h"
+#include "move_funcs_order.h"
 
-#ifdef DMALLOC
-#include "dmalloc.h"
-#endif
+#include "inline.h"
 
-enum fcs_presets_ids
+enum
 {
     FCS_PRESET_BAKERS_DOZEN,
     FCS_PRESET_BAKERS_GAME,
@@ -36,7 +55,7 @@ enum fcs_presets_ids
     FCS_PRESET_SEAHAVEN_TOWERS,
     FCS_PRESET_SIMPLE_SIMON,
     FCS_PRESET_YUKON,
-    FCS_PRESET_BELEAGUERED_CASTLE,
+    FCS_PRESET_BELEAGUERED_CASTLE
 };
 
 static const fcs_preset_t fcs_presets[16] =
@@ -248,16 +267,14 @@ static const fcs_preset_t fcs_presets[16] =
 
         "abcdefgh",
         "abcdefgh",
-    },
+    }
 };
 
-struct fcs_preset_name_struct
+typedef struct
 {
     const char name[32];
     int preset_id;
-};
-
-typedef struct fcs_preset_name_struct fcs_preset_name_t;
+} fcs_preset_name_t;
 
 static const fcs_preset_name_t fcs_preset_names[23] =
 {
@@ -355,7 +372,7 @@ static const fcs_preset_name_t fcs_preset_names[23] =
     },
 };
 
-static int fcs_get_preset_id_by_name(
+static GCC_INLINE int fcs_get_preset_id_by_name(
     const char * name
 )
 {
@@ -376,116 +393,6 @@ static int fcs_get_preset_id_by_name(
     return ret;
 }
 
-static int fc_solve_char_to_test_num(char c)
-{
-    if ((c >= '0') && (c <= '9'))
-    {
-        return c-'0';
-    }
-    else if ((c >= 'a') && (c <= 'h'))
-    {
-        return c-'a'+10;
-    }
-    else if ((c >= 'A') && (c <= 'Z'))
-    {
-        return c-'A'+18;
-    }
-    else
-    {
-        return 0;
-    }
-}
-
-#ifndef min
-#define min(a,b) (((a)<(b))?(a):(b))
-#endif
-
-struct internal_tests_order_struct
-{
-    int tests_order_num;
-    int tests_order[FCS_TESTS_NUM];
-};
-
-typedef struct internal_tests_order_struct internal_tests_order_t;
-
-int fc_solve_apply_tests_order(
-    fcs_tests_order_t * tests_order,
-    const char * string,
-    char * * error_string
-    )
-
-{
-    int a;
-    int len;
-    int test_index;
-    int is_group, is_start_group;
-    if (tests_order->tests)
-    {
-        free(tests_order->tests);
-        tests_order->max_num = 10;
-        tests_order->num = 0;
-        tests_order->tests = malloc(sizeof(tests_order->tests[0])*tests_order->max_num );
-    }
-
-#if 0
-    instance->tests_order_num = min(strlen(string), FCS_TESTS_NUM);
-#endif
-    len = strlen(string);
-    test_index = 0;
-    is_group = 0;
-    is_start_group = 0;
-    for(a=0;(a<len) ;a++)
-    {
-        if ((string[a] == '(') || (string[a] == '['))
-        {
-            if (is_group)
-            {
-                *error_string = strdup("There's a nested random group.");
-                return 1;
-            }
-            is_group = 1;
-            is_start_group = 1;
-            continue;
-        }
-
-        if ((string[a] == ')') || (string[a] == ']'))
-        {
-            if (is_start_group)
-            {
-                *error_string = strdup("There's an empty group.");
-                return 2;
-            }
-            if (! is_group)
-            {
-                *error_string = strdup("There's a renegade right parenthesis or bracket.");
-                return 3;
-            }
-            is_group = 0;
-            is_start_group = 0;
-            continue;
-        }
-        if (test_index == tests_order->max_num)
-        {
-            tests_order->max_num += 10;
-            tests_order->tests = realloc(tests_order->tests, sizeof(tests_order->tests[0]) * tests_order->max_num);
-        }
-        tests_order->tests[test_index] = (fc_solve_char_to_test_num(string[a])%FCS_TESTS_NUM) | (is_group ? FCS_TEST_ORDER_FLAG_RANDOM : 0) | (is_start_group ? FCS_TEST_ORDER_FLAG_START_RANDOM_GROUP : 0);
-
-        test_index++;
-        is_start_group = 0;
-    }
-    if (a != len)
-    {
-        *error_string = strdup("The Input string is too long.");
-        return 4;
-    }
-
-    tests_order->num = test_index;
-    *error_string = NULL;
-
-    return 0;
-}
-
 int fc_solve_apply_preset_by_ptr(
     fc_solve_instance_t * instance,
     const fcs_preset_t * preset_ptr
@@ -527,6 +434,7 @@ int fc_solve_apply_preset_by_ptr(
 
     {
         int ht_idx, st_idx;
+        char test_name[2] = {0};
         for(ht_idx = 0; ht_idx < instance->num_hard_threads ; ht_idx++)
         {
             for(st_idx = 0; st_idx < instance->hard_threads[ht_idx]->num_soft_threads; st_idx++)
@@ -542,8 +450,9 @@ int fc_solve_apply_preset_by_ptr(
                 {
                     for(s = preset.allowed_tests;*s != '\0';s++)
                     {
+                        test_name[0] = *s;
                         /* Check if this test corresponds to this character */
-                        if ((soft_thread->tests_order.tests[num_valid_tests] & FCS_TEST_ORDER_NO_FLAGS_MASK) == ((fc_solve_char_to_test_num(*s)%FCS_TESTS_NUM)))
+                        if ((soft_thread->tests_order.tests[num_valid_tests] & FCS_TEST_ORDER_NO_FLAGS_MASK) == ((fc_solve_string_to_test_num(test_name)%FCS_TESTS_NUM)))
                         {
                             break;
                         }
@@ -578,7 +487,7 @@ int fc_solve_apply_preset_by_ptr(
     return FCS_PRESET_CODE_OK;
 }
 
-static int fcs_get_preset_by_id(
+static GCC_INLINE int fcs_get_preset_by_id(
     int preset_id,
     const fcs_preset_t * * preset_ptr
     )
